@@ -5,11 +5,12 @@
 //!
 //! See: <https://docs.github.com/en/rest/security-advisories/global-advisories?apiVersion=2022-11-28>
 
-use anyhow::{Result, anyhow};
+use anyhow::anyhow;
 use github_actions_models::common::{RepositoryUses, Uses};
 
 use super::{Audit, AuditLoadError, audit_meta};
 use crate::{
+    audit::AuditError,
     config::Config,
     finding::{Confidence, Finding, Fix, Severity, location::Routable as _},
     github,
@@ -29,10 +30,10 @@ audit_meta!(
 );
 
 impl KnownVulnerableActions {
-    fn action_known_vulnerabilities(
+    async fn action_known_vulnerabilities(
         &self,
         uses: &RepositoryUses,
-    ) -> Result<Vec<(Severity, String, Option<String>)>> {
+    ) -> Result<Vec<(Severity, String, Option<String>)>, AuditError> {
         let version = match &uses.git_ref {
             // If `uses` is pinned to a symbolic ref, we need to perform
             // feats of heroism to figure out what's going on.
@@ -51,9 +52,11 @@ impl KnownVulnerableActions {
             // To handle all of the above, we convert the ref into a commit
             // and then find the longest tag for that commit.
             version if !uses.ref_is_commit() => {
-                let Some(commit_ref) =
-                    self.client
-                        .commit_for_ref(&uses.owner, &uses.repo, version)?
+                let Some(commit_ref) = self
+                    .client
+                    .commit_for_ref(&uses.owner, &uses.repo, version)
+                    .await
+                    .map_err(Self::err)?
                 else {
                     // No `ref -> commit` means that the action's version
                     // is probably just outright invalid.
@@ -62,7 +65,9 @@ impl KnownVulnerableActions {
 
                 match self
                     .client
-                    .longest_tag_for_commit(&uses.owner, &uses.repo, &commit_ref)?
+                    .longest_tag_for_commit(&uses.owner, &uses.repo, &commit_ref)
+                    .await
+                    .map_err(Self::err)?
                 {
                     Some(tag) => tag.name,
                     // Somehow we've round-tripped through a commit and ended
@@ -79,7 +84,9 @@ impl KnownVulnerableActions {
             commit_ref => {
                 match self
                     .client
-                    .longest_tag_for_commit(&uses.owner, &uses.repo, commit_ref)?
+                    .longest_tag_for_commit(&uses.owner, &uses.repo, commit_ref)
+                    .await
+                    .map_err(Self::err)?
                 {
                     Some(tag) => tag.name,
                     // No corresponding tag means the user is maybe doing something
@@ -93,7 +100,9 @@ impl KnownVulnerableActions {
 
         let vulns = self
             .client
-            .gha_advisories(&uses.owner, &uses.repo, &version)?;
+            .gha_advisories(&uses.owner, &uses.repo, &version)
+            .await
+            .map_err(Self::err)?;
 
         let mut results = vec![];
 
@@ -120,12 +129,12 @@ impl KnownVulnerableActions {
     }
 
     /// Create a fix to upgrade to a specific non-vulnerable version
-    fn create_upgrade_fix<'doc>(
+    async fn create_upgrade_fix<'doc>(
         &self,
         uses: &RepositoryUses,
         target_version: String,
         step: &impl StepCommon<'doc>,
-    ) -> Result<Fix<'doc>> {
+    ) -> Result<Fix<'doc>, AuditError> {
         let mut uses_slug = format!("{}/{}", uses.owner, uses.repo);
         if let Some(subpath) = &uses.subpath {
             uses_slug.push_str(&format!("/{subpath}"));
@@ -150,22 +159,27 @@ impl KnownVulnerableActions {
                 // convention or not, so we have to try both.
                 // We try the prefixed version first, since we expect it
                 // to be more common.
-                let (target_ref, target_commit) = self
+
+                let (target_ref, target_commit) = match self
                     .client
                     .commit_for_ref(&uses.owner, &uses.repo, &prefixed_version)
-                    .map(|commit| commit.map(|commit| (&prefixed_version, commit)))
-                    .or_else(|_| {
-                        self.client
-                            .commit_for_ref(&uses.owner, &uses.repo, &bare_version)
-                            .map(|commit| commit.map(|commit| (&bare_version, commit)))
-                    })?
-                    .ok_or_else(|| {
-                        anyhow!(
-                            "Cannot resolve version {bare_version} to commit hash for {}/{}",
-                            uses.owner,
-                            uses.repo
-                        )
-                    })?;
+                    .await
+                {
+                    Ok(commit) => commit.map(|commit| (&prefixed_version, commit)),
+                    Err(_) => self
+                        .client
+                        .commit_for_ref(&uses.owner, &uses.repo, &bare_version)
+                        .await
+                        .map_err(Self::err)?
+                        .map(|commit| (&bare_version, commit)),
+                }
+                .ok_or_else(|| {
+                    Self::err(anyhow!(
+                        "Cannot resolve version {bare_version} to commit hash for {}/{}",
+                        uses.owner,
+                        uses.repo
+                    ))
+                })?;
 
                 let new_uses_value = format!("{uses_slug}@{target_commit}");
 
@@ -214,14 +228,18 @@ impl KnownVulnerableActions {
         }
     }
 
-    fn process_step<'doc>(&self, step: &impl StepCommon<'doc>) -> Result<Vec<Finding<'doc>>> {
+    async fn process_step<'doc>(
+        &self,
+        step: &impl StepCommon<'doc>,
+    ) -> Result<Vec<Finding<'doc>>, AuditError> {
         let mut findings = vec![];
 
         let Some(Uses::Repository(uses)) = step.uses() else {
             return Ok(findings);
         };
 
-        for (severity, id, first_patched_version) in self.action_known_vulnerabilities(uses)? {
+        for (severity, id, first_patched_version) in self.action_known_vulnerabilities(uses).await?
+        {
             let mut finding_builder = Self::finding()
                 .confidence(Confidence::High)
                 .severity(severity)
@@ -243,20 +261,21 @@ impl KnownVulnerableActions {
             // and only apply the highest one. This would be moderately annoying
             // to do, since we'd have to decide which finding to attach that
             // fix to.
-            if let Some(fix) = first_patched_version
-                .map(|patched_version| self.create_upgrade_fix(uses, patched_version, step))
-                .transpose()?
-            {
+            if let Some(first_patched_version) = first_patched_version {
+                let fix = self
+                    .create_upgrade_fix(uses, first_patched_version, step)
+                    .await?;
                 finding_builder = finding_builder.fix(fix);
             }
 
-            findings.push(finding_builder.build(step)?);
+            findings.push(finding_builder.build(step).map_err(Self::err)?);
         }
 
         Ok(findings)
     }
 }
 
+#[async_trait::async_trait]
 impl Audit for KnownVulnerableActions {
     fn new(state: &AuditState) -> Result<Self, AuditLoadError>
     where
@@ -275,16 +294,20 @@ impl Audit for KnownVulnerableActions {
             .map(|client| KnownVulnerableActions { client })
     }
 
-    fn audit_step<'doc>(&self, step: &Step<'doc>, _config: &Config) -> Result<Vec<Finding<'doc>>> {
-        self.process_step(step)
+    async fn audit_step<'doc>(
+        &self,
+        step: &Step<'doc>,
+        _config: &Config,
+    ) -> Result<Vec<Finding<'doc>>, AuditError> {
+        self.process_step(step).await
     }
 
-    fn audit_composite_step<'doc>(
+    async fn audit_composite_step<'doc>(
         &self,
         step: &CompositeStep<'doc>,
         _config: &Config,
-    ) -> Result<Vec<Finding<'doc>>> {
-        self.process_step(step)
+    ) -> Result<Vec<Finding<'doc>>, AuditError> {
+        self.process_step(step).await
     }
 }
 
@@ -314,8 +337,8 @@ mod tests {
         KnownVulnerableActions::new(&state).unwrap()
     }
 
-    #[test]
-    fn test_fix_upgrade_actions_checkout() {
+    #[tokio::test]
+    async fn test_fix_upgrade_actions_checkout() {
         let workflow_content = r#"
 name: Test Vulnerable Actions
 on: push
@@ -329,7 +352,7 @@ jobs:
         run: echo "hello"
 "#;
 
-        let key = InputKey::local("fakegroup".into(), "test_checkout.yml", None::<&str>).unwrap();
+        let key = InputKey::local("fakegroup".into(), "test_checkout.yml", None::<&str>);
         let workflow = Workflow::from_string(workflow_content.to_string(), key).unwrap();
         let job = workflow.jobs().next().unwrap();
         let steps: Vec<_> = match job {
@@ -347,7 +370,10 @@ jobs:
         };
 
         let audit = create_test_audit();
-        let fix = audit.create_upgrade_fix(&uses, "v4".into(), step).unwrap();
+        let fix = audit
+            .create_upgrade_fix(&uses, "v4".into(), step)
+            .await
+            .unwrap();
         let fixed_document = fix.apply(workflow.as_document()).unwrap();
 
         insta::assert_snapshot!(fixed_document.source(), @r#"
@@ -364,8 +390,8 @@ jobs:
         "#);
     }
 
-    #[test]
-    fn test_fix_upgrade_actions_setup_node() {
+    #[tokio::test]
+    async fn test_fix_upgrade_actions_setup_node() {
         let workflow_content = r#"
 name: Test Node Setup
 on: push
@@ -381,7 +407,7 @@ jobs:
         run: npm install
 "#;
 
-        let key = InputKey::local("fakegroup".into(), "test_setup_node.yml", None::<&str>).unwrap();
+        let key = InputKey::local("fakegroup".into(), "test_setup_node.yml", None::<&str>);
         let workflow = Workflow::from_string(workflow_content.to_string(), key).unwrap();
         let job = workflow.jobs().next().unwrap();
         let steps: Vec<_> = match job {
@@ -399,7 +425,10 @@ jobs:
         };
 
         let audit = create_test_audit();
-        let fix = audit.create_upgrade_fix(&uses, "v4".into(), step).unwrap();
+        let fix = audit
+            .create_upgrade_fix(&uses, "v4".into(), step)
+            .await
+            .unwrap();
         let fixed_document = fix.apply(workflow.as_document()).unwrap();
 
         insta::assert_snapshot!(fixed_document.source(), @r#"
@@ -418,8 +447,8 @@ jobs:
         "#);
     }
 
-    #[test]
-    fn test_fix_upgrade_third_party_action() {
+    #[tokio::test]
+    async fn test_fix_upgrade_third_party_action() {
         let workflow_content = r#"
 name: Test Third Party Action
 on: push
@@ -435,8 +464,7 @@ jobs:
         run: echo "test"
 "#;
 
-        let key =
-            InputKey::local("fakegroup".into(), "test_third_party.yml", None::<&str>).unwrap();
+        let key = InputKey::local("fakegroup".into(), "test_third_party.yml", None::<&str>);
         let workflow = Workflow::from_string(workflow_content.to_string(), key).unwrap();
         let job = workflow.jobs().next().unwrap();
         let steps: Vec<_> = match job {
@@ -454,7 +482,10 @@ jobs:
         };
 
         let audit = create_test_audit();
-        let fix = audit.create_upgrade_fix(&uses, "v4".into(), step).unwrap();
+        let fix = audit
+            .create_upgrade_fix(&uses, "v4".into(), step)
+            .await
+            .unwrap();
         let fixed_document = fix.apply(workflow.as_document()).unwrap();
 
         insta::assert_snapshot!(fixed_document.source(), @r#"
@@ -473,8 +504,8 @@ jobs:
         "#);
     }
 
-    #[test]
-    fn test_fix_upgrade_multiple_vulnerable_actions() {
+    #[tokio::test]
+    async fn test_fix_upgrade_multiple_vulnerable_actions() {
         let workflow_content = r#"
 name: Test Multiple Vulnerable Actions
 on: push
@@ -497,7 +528,7 @@ jobs:
         run: npm install
 "#;
 
-        let key = InputKey::local("fakegroup".into(), "test_multiple.yml", None::<&str>).unwrap();
+        let key = InputKey::local("fakegroup".into(), "test_multiple.yml", None::<&str>);
         let workflow = Workflow::from_string(workflow_content.to_string(), key).unwrap();
         let job = workflow.jobs().next().unwrap();
         let steps: Vec<_> = match job {
@@ -518,6 +549,7 @@ jobs:
         };
         let fix_checkout = audit
             .create_upgrade_fix(&uses_checkout, "v4".into(), &steps[0])
+            .await
             .unwrap();
         current_document = fix_checkout.apply(&current_document).unwrap();
 
@@ -530,6 +562,7 @@ jobs:
         };
         let fix_node = audit
             .create_upgrade_fix(&uses_node, "v4".into(), &steps[1])
+            .await
             .unwrap();
         current_document = fix_node.apply(&current_document).unwrap();
 
@@ -542,6 +575,7 @@ jobs:
         };
         let fix_cache = audit
             .create_upgrade_fix(&uses_cache, "v4".into(), &steps[2])
+            .await
             .unwrap();
         current_document = fix_cache.apply(&current_document).unwrap();
 
@@ -568,8 +602,8 @@ jobs:
         "#);
     }
 
-    #[test]
-    fn test_fix_upgrade_action_with_subpath() {
+    #[tokio::test]
+    async fn test_fix_upgrade_action_with_subpath() {
         let workflow_content = r#"
 name: Test Action with Subpath
 on: push
@@ -583,7 +617,7 @@ jobs:
           param: value
 "#;
 
-        let key = InputKey::local("fakegroup".into(), "test_subpath.yml", None::<&str>).unwrap();
+        let key = InputKey::local("fakegroup".into(), "test_subpath.yml", None::<&str>);
         let workflow = Workflow::from_string(workflow_content.to_string(), key).unwrap();
         let job = workflow.jobs().next().unwrap();
         let steps: Vec<_> = match job {
@@ -601,7 +635,10 @@ jobs:
         };
 
         let audit = create_test_audit();
-        let fix = audit.create_upgrade_fix(&uses, "v2".into(), step).unwrap();
+        let fix = audit
+            .create_upgrade_fix(&uses, "v2".into(), step)
+            .await
+            .unwrap();
         let fixed_document = fix.apply(workflow.as_document()).unwrap();
 
         insta::assert_snapshot!(fixed_document.source(), @r"
@@ -618,8 +655,8 @@ jobs:
         ");
     }
 
-    #[test]
-    fn test_first_patched_version_priority() {
+    #[tokio::test]
+    async fn test_first_patched_version_priority() {
         // This test verifies that first_patched_version is used when available
         let workflow_content = r#"
 name: Test First Patched Version Priority
@@ -632,8 +669,7 @@ jobs:
         uses: actions/checkout@v2
 "#;
 
-        let key =
-            InputKey::local("fakegroup".into(), "test_first_patched.yml", None::<&str>).unwrap();
+        let key = InputKey::local("fakegroup".into(), "test_first_patched.yml", None::<&str>);
         let workflow = Workflow::from_string(workflow_content.to_string(), key).unwrap();
         let job = workflow.jobs().next().unwrap();
         let steps: Vec<_> = match job {
@@ -653,6 +689,7 @@ jobs:
         let audit = create_test_audit();
         let fix_with_patched_version = audit
             .create_upgrade_fix(&uses, "v3.1.0".into(), step)
+            .await
             .unwrap();
         let fixed_document = fix_with_patched_version
             .apply(workflow.as_document())
@@ -670,8 +707,8 @@ jobs:
         "#);
     }
 
-    #[test]
-    fn test_fix_symbolic_ref() {
+    #[tokio::test]
+    async fn test_fix_symbolic_ref() {
         let workflow_content = r#"
 name: Test Non-Commit Ref
 on: push
@@ -683,7 +720,7 @@ jobs:
         uses: actions/checkout@v2 # this comment stays
 "#;
 
-        let key = InputKey::local("fakegroup".into(), "test_non_commit.yml", None::<&str>).unwrap();
+        let key = InputKey::local("fakegroup".into(), "test_non_commit.yml", None::<&str>);
         let workflow = Workflow::from_string(workflow_content.to_string(), key).unwrap();
         let job = workflow.jobs().next().unwrap();
         let steps: Vec<_> = match job {
@@ -700,7 +737,10 @@ jobs:
         };
 
         let audit = create_test_audit();
-        let fix = audit.create_upgrade_fix(&uses, "v4".into(), step).unwrap();
+        let fix = audit
+            .create_upgrade_fix(&uses, "v4".into(), step)
+            .await
+            .unwrap();
 
         let new_doc = fix.apply(workflow.as_document()).unwrap();
 
@@ -716,8 +756,8 @@ jobs:
         ");
     }
 
-    #[test]
-    fn test_offline_audit_state_creation() {
+    #[tokio::test]
+    async fn test_offline_audit_state_creation() {
         // Test that we can create an audit state without a GitHub token
         let state = crate::state::AuditState::default();
 
@@ -727,8 +767,8 @@ jobs:
     }
 
     #[cfg(feature = "gh-token-tests")]
-    #[test]
-    fn test_fix_commit_pin() {
+    #[tokio::test]
+    async fn test_fix_commit_pin() {
         // Test with real GitHub API - requires GH_TOKEN environment variable
         let workflow_content = r#"
 name: Test Commit Hash Pinning Real API
@@ -742,7 +782,7 @@ jobs:
         uses: actions/download-artifact@7a1cd3216ca9260cd8022db641d960b1db4d1be4  # v4.0.0
 "#;
 
-        let key = InputKey::local("fakegroup".into(), "dummy.yml", None::<&str>).unwrap();
+        let key = InputKey::local("fakegroup".into(), "dummy.yml", None::<&str>);
         let workflow = Workflow::from_string(workflow_content.to_string(), key).unwrap();
 
         let state = crate::state::AuditState::new(
@@ -762,6 +802,7 @@ jobs:
         let input = workflow.into();
         let findings = audit
             .audit(KnownVulnerableActions::ident(), &input, &Config::default())
+            .await
             .unwrap();
         assert_eq!(findings.len(), 1);
 
@@ -782,8 +823,8 @@ jobs:
     // TODO: test_fix_commit_pin_subpath
 
     #[cfg(feature = "gh-token-tests")]
-    #[test]
-    fn test_fix_commit_pin_no_comment() {
+    #[tokio::test]
+    async fn test_fix_commit_pin_no_comment() {
         // Ensure that we don't rewrite a version comment
         // if the `uses:` clause doesn't already have one.
         let workflow_content = r#"
@@ -797,7 +838,7 @@ jobs:
       - name: Commit pinned action
         uses: actions/download-artifact@7a1cd3216ca9260cd8022db641d960b1db4d1be4
 "#;
-        let key = InputKey::local("fakegroup".into(), "dummy.yml", None::<&str>).unwrap();
+        let key = InputKey::local("fakegroup".into(), "dummy.yml", None::<&str>);
         let workflow = Workflow::from_string(workflow_content.to_string(), key).unwrap();
 
         let state = crate::state::AuditState::new(
@@ -817,6 +858,7 @@ jobs:
         let input = workflow.into();
         let findings = audit
             .audit(KnownVulnerableActions::ident(), &input, &Config::default())
+            .await
             .unwrap();
         assert_eq!(findings.len(), 1);
 
