@@ -1,4 +1,4 @@
-use std::{collections::HashMap, fs, num::NonZeroUsize, str::FromStr};
+use std::{collections::HashMap, fs, num::NonZeroUsize, ops::Deref, str::FromStr};
 
 use anyhow::{Context as _, anyhow};
 use camino::Utf8Path;
@@ -11,7 +11,10 @@ use thiserror::Error;
 
 use crate::{
     App, CollectionOptions,
-    audit::{AuditCore, forbidden_uses::ForbiddenUses, unpinned_uses::UnpinnedUses},
+    audit::{
+        AuditCore, dependabot_cooldown::DependabotCooldown, forbidden_uses::ForbiddenUses,
+        unpinned_uses::UnpinnedUses,
+    },
     finding::Finding,
     github::{Client, ClientError},
     models::uses::RepositoryUsesPattern,
@@ -152,10 +155,46 @@ impl RawConfig {
 }
 
 #[derive(Clone, Debug, Deserialize)]
-#[serde(rename_all = "kebab-case", untagged)]
-pub(crate) enum ForbiddenUsesConfig {
-    Allow { allow: Vec<RepositoryUsesPattern> },
-    Deny { deny: Vec<RepositoryUsesPattern> },
+#[serde(rename_all = "kebab-case")]
+pub(crate) struct DependabotCooldownConfig {
+    pub(crate) days: NonZeroUsize,
+}
+
+impl Default for DependabotCooldownConfig {
+    fn default() -> Self {
+        Self {
+            days: NonZeroUsize::new(7).expect("impossible"),
+        }
+    }
+}
+
+/// Slightly annoying wrapper for [`ForbiddenUsesConfigInner`], which is our
+/// real configuration type for the `forbidden-uses` rule.
+///
+/// We need this wrapper type so that we can apply the `singleton_map`
+/// deserializer to the inner type, ensuring that we deserialize from a
+/// mapping with an explicit key discriminant (i.e. `allow:` or `deny:`)
+/// rather than a YAML tag. We could work around this by using serde's
+/// `untagged` instead, but this produces suboptimal user-facing error messages.
+#[derive(Clone, Debug, Deserialize)]
+#[serde(transparent)]
+pub(crate) struct ForbiddenUsesConfig(
+    #[serde(with = "serde_yaml::with::singleton_map")] pub(crate) ForbiddenUsesConfigInner,
+);
+
+impl Deref for ForbiddenUsesConfig {
+    type Target = ForbiddenUsesConfigInner;
+
+    fn deref(&self) -> &Self::Target {
+        &self.0
+    }
+}
+
+#[derive(Clone, Debug, Deserialize)]
+#[serde(rename_all = "kebab-case")]
+pub(crate) enum ForbiddenUsesConfigInner {
+    Allow(Vec<RepositoryUsesPattern>),
+    Deny(Vec<RepositoryUsesPattern>),
 }
 
 /// Config for the `unpinned-uses` rule.
@@ -344,6 +383,7 @@ impl TryFrom<UnpinnedUsesConfig> for UnpinnedUsesPolicies {
 #[derive(Clone, Debug, Default)]
 pub(crate) struct Config {
     raw: RawConfig,
+    pub(crate) dependabot_cooldown_config: DependabotCooldownConfig,
     pub(crate) forbidden_uses_config: Option<ForbiddenUsesConfig>,
     pub(crate) unpinned_uses_policies: UnpinnedUsesPolicies,
 }
@@ -353,6 +393,10 @@ impl Config {
     fn load(contents: &str) -> Result<Self, ConfigErrorInner> {
         let raw = RawConfig::load(contents)?;
 
+        let dependabot_cooldown_config = raw
+            .rule_config(DependabotCooldown::ident())?
+            .unwrap_or_default();
+
         let forbidden_uses_config = raw.rule_config(ForbiddenUses::ident())?;
 
         let unpinned_uses_policies = {
@@ -367,6 +411,7 @@ impl Config {
 
         Ok(Self {
             raw,
+            dependabot_cooldown_config,
             forbidden_uses_config,
             unpinned_uses_policies,
         })
@@ -383,12 +428,12 @@ impl Config {
     ///    to discover a config file. This function is typically one
     ///    of [`Config::discover_local`] or [`Config::discover_remote`]
     ///    depending on the input type.
-    pub(crate) fn discover<F>(
+    pub(crate) async fn discover<F>(
         options: &CollectionOptions,
         discover_fn: F,
     ) -> Result<Self, ConfigError>
     where
-        F: FnOnce() -> Result<Option<Self>, ConfigError>,
+        F: AsyncFnOnce() -> Result<Option<Self>, ConfigError>,
     {
         if options.no_config {
             // User has explicitly disabled config loading.
@@ -401,7 +446,7 @@ impl Config {
             Ok(config.clone())
         } else {
             // Attempt to discover a config file using the provided function.
-            discover_fn().map(|conf| conf.unwrap_or_default())
+            discover_fn().await.map(|conf| conf.unwrap_or_default())
         }
     }
 
@@ -464,7 +509,7 @@ impl Config {
     ///
     /// For directories, this attempts to find a `.github/zizmor.yml` or
     /// `zizmor.yml` in the directory itself.
-    pub(crate) fn discover_local(path: &Utf8Path) -> Result<Option<Self>, ConfigError> {
+    pub(crate) async fn discover_local(path: &Utf8Path) -> Result<Option<Self>, ConfigError> {
         tracing::debug!("discovering config for local input `{path}`");
 
         if path.is_dir() {
@@ -497,37 +542,34 @@ impl Config {
     ///
     /// This will look for a `.github/zizmor.yml` or `zizmor.yml`
     /// in the repository's root directory.
-    pub(crate) fn discover_remote(
+    pub(crate) async fn discover_remote(
         client: &Client,
         slug: &RepoSlug,
     ) -> Result<Option<Self>, ConfigError> {
-        let conf = CONFIG_CANDIDATES
-            .iter()
-            .find_map(|candidate| {
-                client
-                    .fetch_single_file(slug, candidate)
-                    .map_err(|err| ConfigError {
+        for candidate in CONFIG_CANDIDATES {
+            match client.fetch_single_file(slug, candidate).await {
+                Ok(Some(contents)) => {
+                    tracing::debug!("retrieved config candidate `{candidate}` for {slug}");
+
+                    return Some(Self::load(&contents).map_err(|err| ConfigError {
+                        path: candidate.to_string(),
+                        source: err,
+                    }))
+                    .transpose();
+                }
+                Ok(None) => {
+                    continue;
+                }
+                Err(err) => {
+                    return Err(ConfigError {
                         path: candidate.to_string(),
                         source: err.into(),
-                    })
-                    .and_then(|contents| {
-                        contents
-                            .map(|contents| {
-                                tracing::debug!(
-                                    "retrieved config candidate `{candidate}` for {slug}"
-                                );
-                                Self::load(&contents).map_err(|err| ConfigError {
-                                    path: candidate.to_string(),
-                                    source: err,
-                                })
-                            })
-                            .transpose()
-                    })
-                    .transpose()
-            })
-            .transpose()?;
+                    });
+                }
+            }
+        }
 
-        Ok(conf)
+        Ok(None)
     }
 
     /// Loads a global [`Config`] for the given [`App`].
