@@ -3,7 +3,7 @@ use camino::Utf8PathBuf;
 use regex::{Captures, Regex};
 use std::{env::current_dir, io::ErrorKind, sync::LazyLock};
 
-use assert_cmd::Command;
+use assert_cmd::{Command, cargo};
 
 static TEST_PREFIX: LazyLock<Utf8PathBuf> = LazyLock::new(|| {
     let current_dir = current_dir().expect("Cannot figure out current directory");
@@ -39,6 +39,7 @@ pub enum OutputMode {
 
 pub struct Zizmor {
     cmd: Command,
+    stdin: Option<String>,
     unbuffer: bool,
     offline: bool,
     inputs: Vec<String>,
@@ -51,10 +52,11 @@ pub struct Zizmor {
 impl Zizmor {
     /// Create a new zizmor runner.
     pub fn new() -> Self {
-        let cmd = Command::cargo_bin("zizmor").unwrap();
+        let cmd = Command::new(cargo::cargo_bin!());
 
         Self {
             cmd,
+            stdin: None,
             unbuffer: false,
             offline: true,
             inputs: vec![],
@@ -65,6 +67,11 @@ impl Zizmor {
         }
     }
 
+    pub fn stdin(mut self, input: impl Into<String>) -> Self {
+        self.stdin = Some(input.into());
+        self
+    }
+
     pub fn args<'a>(mut self, args: impl IntoIterator<Item = &'a str>) -> Self {
         self.cmd.args(args);
         self
@@ -124,6 +131,10 @@ impl Zizmor {
     }
 
     pub fn run(mut self) -> Result<String> {
+        if let Some(stdin) = &self.stdin {
+            self.cmd.write_stdin(stdin.as_bytes());
+        }
+
         if self.offline {
             self.cmd.arg("--offline");
         } else {
@@ -144,6 +155,16 @@ impl Zizmor {
             self.cmd.arg("--config").arg(config);
         }
 
+        if !self.unbuffer {
+            // NOTE(ww): We explicitly disable progress bars in test runs
+            // because of a tracing-indicatif bug that surfaces when we have
+            // multiple spans and no terminal.
+            // We only hit this when not using `unbuffer`, because `unbuffer`
+            // simulates a TTY.
+            // See: https://github.com/emersonford/tracing-indicatif/issues/24
+            self.cmd.arg("--no-progress");
+        }
+
         for input in &self.inputs {
             self.cmd.arg(input);
         }
@@ -188,8 +209,8 @@ impl Zizmor {
 
         if let Some(exit_code) = output.status.code() {
             // There are other nonzero exit codes that don't indicate failure;
-            // 1 and 2 do.
-            let is_failure = matches!(exit_code, 1 | 2);
+            // these do. 1/2 are general errors, 101 is Rust's panic exit code.
+            let is_failure = matches!(exit_code, 1 | 2 | 101);
             if is_failure != self.expects_failure {
                 anyhow::bail!("zizmor exited with unexpected code {exit_code}: {raw}");
             }
